@@ -1,5 +1,6 @@
+'Scrypt for Python'
 
-__version__ = '1.0.0'
+__version__ = '1.0.1'
 
 # First, try loading libscrypt
 _done = False
@@ -32,3 +33,10 @@ if not _done:
 if not _done:
     from pypyscrypt import *
 
+__all__ = ['scrypt', 'scrypt_mcf', 'scrypt_mcf_check']
+
+# Clean up pydoc output
+del __path__
+del consts
+del pylibscrypt
+
